unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ZylSerialPort, ExtCtrls, Menus;

type
  TMainForm = class(TForm)
    ZylSerialPort: TZylSerialPort;
    Memo: TMemo;
    btnOpen: TButton;
    efSentStr: TEdit;
    btnSendStr: TButton;
    btnClose: TButton;
    lstPorts: TListBox;
    rgMode: TRadioGroup;
    lstBaudRate: TListBox;
    lblPort: TLabel;
    lblBaudRate: TLabel;
    rbDataWidth: TRadioGroup;
    rbParity: TRadioGroup;
    rbStopBits: TRadioGroup;
    rbHwFlowControl: TRadioGroup;
    rbSwFlowControl: TRadioGroup;
    btnApply: TButton;
    gbEnable: TGroupBox;
    cbDTR: TCheckBox;
    cbRTS: TCheckBox;
    efSentBytes: TEdit;
    btnSendBytes: TButton;
    poplstPorts: TPopupMenu;
    miRefreshPorts1: TMenuItem;
    procedure btnOpenClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ZylSerialPortLineStatusChange(Sender: TObject;
      LineStatus: TLineStatusSet);
    procedure btnSendStrClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ZylSerialPortDisconnect(Sender: TObject; Port: TCommPort);
    procedure ZylSerialPortConnect(Sender: TObject; Port: TCommPort);
    procedure ZylSerialPortReceive(Sender: TObject; Buffer: String);
    procedure ZylSerialPortSend(Sender: TObject; Buffer: String);
    procedure btnApplyClick(Sender: TObject);
    procedure btnSendBytesClick(Sender: TObject);
    procedure efSentBytesKeyPress(Sender: TObject; var Key: Char);
    procedure miRefreshPorts1Click(Sender: TObject);
  private
    { Private declarations }
    procedure ApplySettings;
    procedure GetSerialPorts();
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.btnOpenClick(Sender: TObject);
begin

  ZylSerialPort.Port := ZylSerialPort.StringToCommPort(lstPorts.Items[lstPorts.ItemIndex]);
  //ZylSerialPort.CustomPortName := 'virtual';
  ApplySettings;
  ZylSerialPort.NeedSynchronization := True;
  ZylSerialPort.Open;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ZylSerialPort.Close();
end;

procedure TMainForm.ZylSerialPortLineStatusChange(Sender: TObject;
  LineStatus: TLineStatusSet);
var
  strLine: AnsiString;
begin
  strLine := '';
  if lsCTS in LineStatus then
    strLine := strLine + 'CTS ';
  if lsDSR in LineStatus then
    strLine := strLine + 'DSR ';
  if lsRING in LineStatus then
    strLine := strLine + 'RING ';
  if lsCD in LineStatus then
    strLine := strLine + 'CD ';
  Memo.Lines.Add('Line Change: ' + strLine);
end;

procedure TMainForm.btnSendStrClick(Sender: TObject);
begin
  ZylSerialPort.SendString(efSentStr.Text + #13#10);
end;

procedure TMainForm.btnCloseClick(Sender: TObject);
begin
  ZylSerialPort.Close;
end;

procedure TMainForm.GetSerialPorts();
var
  comPorts: TCommPortSet;
  i: TCommPort;
  nPort: Integer;
begin
  lstPorts.Items.Clear;
  comPorts := ZylSerialPort.GetExistingCommPorts();
  for i := spCOM1 to spCOM254 do
  begin
    if i in comPorts then
      lstPorts.Items.Add(ZylSerialPort.CommPortToString(i));
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  GetSerialPorts();
  lstBaudRate.ItemIndex := lstBaudRate.Items.IndexOf('4800');
end;

procedure TMainForm.ZylSerialPortDisconnect(Sender: TObject; Port: TCommPort);
begin
  Memo.Lines.Add('Disconnected from ' + ZylSerialPort.CommPortToString(Port));
end;

procedure TMainForm.ZylSerialPortConnect(Sender: TObject; Port: TCommPort);
begin
  Memo.Lines.Add('Connected to ' + ZylSerialPort.CommPortToString(Port));
end;

procedure TMainForm.ZylSerialPortReceive(Sender: TObject; Buffer: String);
var
  strBuffer: AnsiString;
  i: Integer;
begin
  if rgMode.ItemIndex = 0 then
    Memo.Lines.Add(ZylSerialPort.ReplaceControlChars(Buffer))
  else if rgMode.ItemIndex = 1 then
  begin
    strBuffer := '';
    for i := 1 to Length(Buffer) do
      strBuffer := strBuffer + IntToStr(Ord(Buffer[i]));
    Memo.Lines.Add(strBuffer)
  end
  else
  begin
    strBuffer := '';
    for i := 1 to Length(Buffer) do
      strBuffer := strBuffer + IntToHex(Ord(Buffer[i]), 1);
    Memo.Lines.Add(strBuffer)
  end;
end;

procedure TMainForm.ZylSerialPortSend(Sender: TObject; Buffer: String);
begin
  Memo.Lines.Add('Sent: ' + Buffer);
end;

procedure TMainForm.ApplySettings;
begin
  ZylSerialPort.BaudRate := ZylSerialPort.IntToBaudRate(StrToInt(lstBaudRate.Items[lstBaudRate.ItemIndex]));
  ZylSerialPort.DataWidth := TDataWidth(rbDataWidth.ItemIndex);
  ZylSerialPort.Parity := TParityBits(rbParity.ItemIndex);
  ZylSerialPort.StopBits := TStopBits(rbStopBits.ItemIndex);
  ZylSerialPort.HwFlowControl := THwFlowControl(rbHwFlowControl.ItemIndex);
  ZylSerialPort.SwFlowControl := TSwFlowControl(rbSwFlowControl.ItemIndex);
  if cbDTR.Checked then
    ZylSerialPort.EnableDTROnOpen := True
  else
    ZylSerialPort.EnableDTROnOpen := False;

  if cbRTS.Checked then
    ZylSerialPort.EnableRTSOnOpen := True
  else
    ZylSerialPort.EnableRTSOnOpen := False;
end;

procedure TMainForm.btnApplyClick(Sender: TObject);
begin
  ApplySettings;
end;

//Send bytes in the following format: #10#0#7 means to send byte 10, byte 0, byte 7 (in decimal)
procedure TMainForm.btnSendBytesClick(Sender: TObject);
var
  lstStr: TStringList;
  sentStr: AnsiString;
  i: Integer;
begin
  lstStr := TStringList.Create;
  ExtractStrings(['#'], [' '], PAnsiChar(efSentBytes.Text), lstStr);
  sentStr := '';
  for i := 0 to lstStr.Count -1 do
  begin
    try
      sentStr := sentStr + Chr(StrToInt(lstStr[i]));
    except
    end;  
  end;
  lstStr.Free;
  ZylSerialPort.SendChars(sentStr);
end;

procedure TMainForm.efSentBytesKeyPress(Sender: TObject; var Key: Char);
begin
  if (Ord(Key) < 48) or (Ord(Key) > 57) then
    if Key <> '#' then
      Key := #0;
end;

procedure TMainForm.miRefreshPorts1Click(Sender: TObject);
begin
  lstPorts.Clear;
  GetSerialPorts;
end;

end.
